package linkate;

import java.util.NoSuchElementException;
import java.util.Iterator;
import java.util.Collection;

public class CodaLinkata<E>
implements java.util.Queue<E>
{
	NodoDoppio<E> head, tail;
	int count;

	public CodaLinkata()
	{
		head = tail = null;
		count = 0;
	}

	public boolean add(E e)
	{
		if(!offer(e))
		{
			throw new IllegalStateException("Non ce la posso fare");
		}
		return true;
	}

	public E element()
	{
		if(isEmpty())
		{
			throw new NoSuchElementException("Vuoto pneumatico");
		}
		return peek();
	}

	public boolean offer(E e)
	{
		NodoDoppio<E> tmp = new NodoDoppio<E>(e);
		
		if(isEmpty())
		{
			head = tail = tmp;
		}
		else
		{
			tail.next = tmp;
			tmp.prev = tail;
			tail = tail.next;
		}
		count++;
		return true;
	}

	public E peek()
	{
		if(isEmpty())
		{
			return null;
		}
		return head.getValue();
	}

	public E poll()
	{
		if(isEmpty())
		{
			return null;
		}
		E retval = head.getValue();

		if(size() == 1)
		{
			head = tail = null;
		}
		else
		{
			head = head.next;
			head.prev.next = null;
			head.prev = null;
		}
		count--;
		return retval;
	}

	public E remove()
	{
		if(isEmpty())
		{
			throw new NoSuchElementException("Vuoto pneumatico");
		}
		return poll();
	}

	public boolean isEmpty()
	{
		return count == 0;
	}

	public int size()
	{
		return count;
	}

	public void clear()
	{
		NodoDoppio<E> tmp = head;
		while(tmp != null)
		{
			tmp.value = null;
			tmp.prev = null;
			tmp = tmp.next;
		}
		count = 0;
		head = tail = null;
	}

	public class IteratoreCodaLinkata<E>
	implements Iterator<E>
	{
		CodaLinkata<E> cl;
		NodoDoppio<E> tmpNext;
		boolean nextDone = false;

		public IteratoreCodaLinkata(CodaLinkata<E> theCL)
		{
			cl = theCL;
			tmpNext = cl.head;
		}

		public boolean hasNext()
		{
			return tmpNext != null;
		}

		public E next()
		{
			E retval = tmpNext.getValue();
			tmpNext = tmpNext.next;
			nextDone = true;
			return retval;
		}

		public void remove()
		{
			if(!nextDone)
			{
				throw new IllegalStateException("Prima next, poi remove");
			}
			if(cl.size() == 1)
			{
				cl.remove();
				return;
			}
			if(tmpNext == null)
			{
				cl.tail = cl.tail.prev;
				cl.tail.next.prev = null;
				cl.tail.next = null;
				cl.count--;
			}
			else if(cl.size() == 2 || tmpNext.prev.prev == null)
			{
				cl.remove();
			}
			else
			{
			}
		}
	}
}
